@@ -51,8 +51,8 @@
 
       <div class="indent">
         <p class="code"><tt>
-        $ <b>cd /usr/src/linux-2.6.11/;</b><br />
-        $ <b>patch -p1 -i /usr/src/ttyrpld/kpatch/linux-2.6.11.diff;</b><br />
+        $ <b>cd /usr/src/linux-2.6.13/;</b><br />
+        $ <b>patch -p1 -i /usr/src/ttyrpld/kpatch/linux-2.6.13.diff;</b><br />
         $ <b>make menuconfig;</b>
         </tt></p>
       </div>
@@ -61,9 +61,6 @@
       suits your needs to choose "<tt>TTY logging via rpldev</tt>" in the
       "<tt>Security options</tt>" menu.</p>
 
-      <p class="block"><i>Note: For Linux Kernels 2.4.x, the "TTY logging via
-      rpldev" option is available in the "Character devices" menu.</p>
-
     </td>
   </tr>
   <tr>
@@ -83,7 +80,7 @@
       <div class="indent">
         <p class="code"><tt>
         $ <b>cd /usr/src/sys/;</b><br />
-        $ <b>patch -p1 -i /usr/src/ttyrpld/kpatch/freebsd-5.3.diff;</b>
+        $ <b>patch -p1 -i /usr/src/ttyrpld/kpatch/freebsd-6.0.diff;</b>
         </tt></p>
       </div>
 
@@ -99,6 +96,9 @@
         </tt></p>
       </div>
 
+      <p class="block">For `<tt>make kernel</tt>` to work, various source
+      packages need to be installed.</p>
+
     </td>
   </tr>
   <tr>
@@ -114,7 +114,7 @@
       <div class="indent">
         <p class="code"><tt>
         $ <b>cd /usr/src/sys/;</b><br />
-        $ <b>patch -p1 -i /usr/src/ttyrpld/kpatch/openbsd-3.6.diff;</b>
+        $ <b>patch -p1 -i /usr/src/ttyrpld/kpatch/openbsd-3.8.diff;</b>
         </tt></p>
       </div>
 
@@ -154,7 +154,7 @@
 
       <p class="block">BSD users have to use <tt>gmake</tt>;
       <tt>KERNEL_DIR</tt> and <tt>MODULES_DIR</tt> is not applicable for BSD,
-      but you may <tt>cd</tt> into <tt>freebsd-5.3</tt> by hand and run (bsd)
+      but you may <tt>cd</tt> into <tt>freebsd-6.0</tt> by hand and run (bsd)
       <tt>make</tt> with any options you need. The kmod target <i>will not</i>
       install the module on FreeBSD.</p>
 
@@ -183,18 +183,19 @@
     <td valign="top">
 
       <p class="block">To compile the user-space tools, run `<tt>make</tt>`
-      without any arguments. This will build <tt>rpld</tt> and
-      <tt>ttyreplay</tt> (the daemon and the log analyzer).</p>
+      (BSD: `<tt>gmake</tt>`) without any arguments. This will build
+      <tt>rpld</tt> and <tt>ttyreplay</tt> (the daemon and the log
+      analyzer).</p>
 
       <p class="block">You will need <i><a
       href="http://jengelh.hopto.org/coding/libHX.php"
       target="_blank">libHX</a></i> to use <i>ttyrpld</i> since it makes use of
       advanced data storage and string functions not present in libc.</p>
 
-      <p class="block">It is recommended that you use <i>gcc3</i>. However,
-      <i>gcc2</i> can also be used with minor modifications to the dependency
-      file generation instructions in the Makefiles.</p> BSD: Expect warnings
-      due to bad-coded libc header files.</p>
+      <p class="block">It is recommended that you use <i>gcc version 3 or
+      version 4</i>. However, <i>gcc2</i> can also be used with minor
+      modifications to the dependency file generation instructions in the
+      Makefiles.</p>
 
     </td>
   </tr>
