@@ -42,9 +42,10 @@ freebsd-5.3/rpldev.c - Kernel interface for RPLD
 #include "../include/rpl_packet.h"
 
 #define IO_NDELAY      0x10 // from sys/vnode.h -- inclusion problems
-#define SKIP_PTM(tty)  if(major(tty->t_dev) == 6) { return 0; }
-#define TTY_DEVNR(tty) htole32(mkdev_26(major(tty->t_dev), minor(tty->t_dev)))
-#define RET_IF_INACT   if(!kmi_usecount) { return 0; }
+#define SKIP_PTM(tty)  if(umajor(dev2udev(tty->t_dev)) == 6) return 0;
+#define TTY_DEVNR(tty) htole32(mkdev_26(umajor(dev2udev(tty->t_dev)), \
+                       uminor(dev2udev(tty->t_dev))))
+#define RET_IF_INACT   if(!kmi_usecount) return 0;
 
 // Module stuff
 static int kmd_event(module_t, int, void *);
@@ -89,7 +90,6 @@ static struct cdev *kmi_node;
 static struct cdevsw kmi_fops = {
     .d_version = D_VERSION,
     .d_name    = "rpldev",
-    .d_maj     = MAJOR_AUTO,
     .d_open    = urpl_open,
     .d_read    = urpl_read,
     .d_ioctl   = urpl_ioctl,
@@ -107,7 +107,8 @@ static int kmd_event(module_t mp, int type, void *data) {
     int eax;
     switch(type) {
         case MOD_LOAD:
-            if((eax = kmd_load()) != 0) { kmd_unload(); }
+            if((eax = kmd_load()) != 0)
+                kmd_unload();
             return eax;
         case MOD_UNLOAD:
             return kmd_unload();
@@ -118,15 +119,16 @@ static int kmd_event(module_t mp, int type, void *data) {
 static int kmd_load(void) {
     mtx_init(&Buffer_lock, "rpldev", NULL, MTX_DEF);
     mtx_init(&Open_lock, "rpldev", NULL, MTX_DEF);
-    if((kmi_node = make_dev(&kmi_fops, 0, UID_ROOT, 0, 0600, "rpl")) == NULL) {
+    if((kmi_node = make_dev(&kmi_fops, 0, UID_ROOT, 0, 0600, "rpl")) == NULL)
         return ENOMEM;
-    }
     return 0;
 }
 
 static int kmd_unload(void) {
-    if(kmi_usecount) { return EBUSY; }
-    if(kmi_node != NULL) { destroy_dev(kmi_node); }
+    if(kmi_usecount)
+        return EBUSY;
+    if(kmi_node != NULL)
+        destroy_dev(kmi_node);
     mtx_destroy(&Buffer_lock);
     mtx_destroy(&Open_lock);
     return 0;
@@ -166,7 +168,7 @@ static int krpl_read(const char *buf, int count, struct tty *tty) {
 
     RET_IF_INACT;
     SKIP_PTM(tty);
-    if(count == 0) { return 0; }
+    if(count == 0) return 0;
 
     p.dev   = TTY_DEVNR(tty);
     p.size  = htole16(count);
@@ -181,7 +183,7 @@ static int krpl_write(const char *buf, int count, struct tty *tty) {
 
     RET_IF_INACT;
     SKIP_PTM(tty);
-    if(count == 0) { return 0; }
+    if(count == 0) return 0;
 
     p.dev   = TTY_DEVNR(tty);
     p.size  = htole16(count);
@@ -279,9 +281,8 @@ static int urpl_read(struct cdev *cd, struct uio *uio, int flags) {
     while(BufRP == BufWP) {
         mtx_unlock(&Buffer_lock);
         if(flags & IO_NDELAY) { return EWOULDBLOCK; }
-        if((eax = tsleep(&Buffer, PCATCH, "rpldev", 0)) != 0) {
+        if((eax = tsleep(&Buffer, PCATCH, "rpldev", 0)) != 0)
             return eax;
-        }
         mtx_lock(&Buffer_lock);
     }
 
@@ -297,8 +298,10 @@ static int urpl_ioctl(struct cdev *cd, u_long cmd, caddr_t data, int flags,
     /* Gah, it does not even support multiple return values >:-( so I need to
     hack around with data. */
     size_t *ptr = (void *)data;
-    if(IOCGROUP(cmd) != RPL_IOC_MAGIC) { return ENOTTY; }
-    if(ptr == NULL) { return EFAULT; }
+    if(IOCGROUP(cmd) != RPL_IOC_MAGIC)
+        return ENOTTY;
+    if(ptr == NULL)
+        return EFAULT;
 
     switch(cmd) {
         case RPL_IOC_GETBUFSIZE:
@@ -315,8 +318,8 @@ static int urpl_ioctl(struct cdev *cd, u_long cmd, caddr_t data, int flags,
             return 0;
         case RPL_IOC_SEEK:
             mtx_lock(&Buffer_lock);
-            BufRP = Buffer + (BufRP - Buffer + min_uint(*ptr,
-              avail_R())) % Bufsize;
+            BufRP = Buffer + (BufRP - Buffer +
+                    min_uint(*ptr, avail_R())) % Bufsize;
             if(BufRP >= Buffer + Bufsize) { 
                 log(LOG_ERR, "BufRP overflow (should not happen!) -"
                  " resetting BufRP/BufWP\n");
@@ -351,29 +354,29 @@ static int urpl_close(struct cdev *cd, int flags, int fmt, struct thread *th) {
 
 //-----------------------------------------------------------------------------
 static inline ssize_t avail_R(void) {
-    if(BufWP >= BufRP) { return BufWP - BufRP; }
+    if(BufWP >= BufRP)
+        return BufWP - BufRP;
     return BufWP + Bufsize - BufRP;
 }
 
 static inline ssize_t avail_W(void) {
-    if(BufWP >= BufRP) { return BufRP + Bufsize - BufWP - 1; }
+    if(BufWP >= BufRP)
+        return BufRP + Bufsize - BufWP - 1;
     return BufRP - BufWP - 1;
 }
 
 static inline void fill_time(struct timeval *tv) {
     microtime(tv);
 
-    if(sizeof(tv->tv_sec) == sizeof(uint32_t)) {
+    if(sizeof(tv->tv_sec) == sizeof(uint32_t))
         tv->tv_sec = htole32(tv->tv_sec);
-    } else if(sizeof(tv->tv_sec) == sizeof(uint64_t)) {
+    else if(sizeof(tv->tv_sec) == sizeof(uint64_t))
         tv->tv_sec = htole64(tv->tv_sec);
-    }
 
-    if(sizeof(tv->tv_usec) == sizeof(uint32_t)) {
+    if(sizeof(tv->tv_usec) == sizeof(uint32_t))
         tv->tv_usec = htole32(tv->tv_usec);
-    } else if(sizeof(tv->tv_usec) == sizeof(uint64_t)) {
+    else if(sizeof(tv->tv_usec) == sizeof(uint64_t))
         tv->tv_usec = htole64(tv->tv_usec);
-    }
     return;
 }
 
@@ -393,7 +396,8 @@ static int mv_buffer(struct rpldev_packet *p, const void *buf, size_t count) {
     }
 
     mv_buffer2(p, sizeof(struct rpldev_packet));
-    if(count > 0) { mv_buffer2(buf, count); }
+    if(count > 0)
+        mv_buffer2(buf, count);
     mtx_unlock(&Buffer_lock);
     wakeup_one(&Buffer);
     return count;
@@ -419,13 +423,13 @@ static inline int mv_to_user(struct uio *uio, size_t count) {
     int eax;
 
     if(count < x) {
-        if((eax = uiomove(BufRP, count, uio))) { return eax; }
+        if((eax = uiomove(BufRP, count, uio)))
+            return eax;
         BufRP += count;
     } else {
         if((eax = uiomove(BufRP, x, uio)) ||
-         (eax = uiomove(Buffer, count - x, uio))) {
+         (eax = uiomove(Buffer, count - x, uio)))
             return eax;
-        }
         BufRP = Buffer + count - x;
     }
 
