@@ -28,22 +28,28 @@ include/rpl_endian.h
 #    include <sys/endian.h>
 #    if _BYTE_ORDER == _BIG_ENDIAN
 #        define SWAB(ptr, sz) swab_be((ptr), (sz))
+#        define SWAB1(ptr) swab_be((ptr), sizeof(*(ptr)))
 #    else
 #        define SWAB(ptr, sz)
+#        define SWAB1(ptr)
 #    endif
 #elif defined(__OpenBSD__)
 #    include <machine/endian.h>
 #    if BYTE_ORDER == BIG_ENDIAN
 #        define SWAB(ptr, sz) swab_be((ptr), (sz))
+#        define SWAB1(ptr) swab_be((ptr), sizeof(*(ptr)))
 #    else
 #        define SWAB(ptr, sz)
+#        define SWAB1(ptr)
 #    endif
 #else
 #    include <endian.h>
 #    if __BYTE_ORDER == __BIG_ENDIAN
 #        define SWAB(ptr, sz) swab_be((ptr), (sz))
+#        define SWAB1(ptr) swab_be((ptr), sizeof(*(ptr)))
 #    else
 #        define SWAB(ptr, sz)
+#        define SWAB1(ptr)
 #    endif
 #endif
 
