@@ -2,6 +2,7 @@
 ttyrpld - TTY replay daemon
 include/rpl_packet.h - Packet definitions for RPLD and RPLDEV
   Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004 - 2005
+  Copyright © Nguyen Anh Quynh <aquynh [at] gmail com>, 2005
   -- License restrictions apply (GPL v2)
 
   This file is part of ttyrpld.
@@ -48,6 +49,8 @@ enum {
     EVT_DEINIT  = 0x64,
     EVT_ID_PROG = 0xF0,
     EVT_ID_DEVPATH,
+    EVT_ID_TIME,
+    EVT_ID_USER,
     EVT_max,
     MAGIC_SIG = 0xEE,
 };
