@@ -29,6 +29,7 @@ kernel/rpldev.c - Kernel interface for RPLD
 #include <linux/init.h>
 #include <linux/kernel.h>
 #include <linux/km_rpldev.h>
+#include <linux/major.h>
 #include <linux/miscdevice.h>
 #include <linux/module.h>
 #include <linux/moduleparam.h>
@@ -49,22 +50,12 @@ kernel/rpldev.c - Kernel interface for RPLD
 #define PREFIX "rpldev: "
 #define MAXFNLEN 256
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
-#    define KERNEL_2_4 1
-#    define IS_PTY_MASTER(tty) \
-      (((tty)->driver.major >= UNIX98_PTY_MASTER_MAJOR && \
-      (tty)->driver.major < UNIX98_PTY_MASTER_MAJOR + \
-      UNIX98_PTY_MAJOR_COUNT) || (tty)->driver.major == PTY_MASTER_MAJOR)
-#    define TTY_DEVNR(tty) \
-      cpu_to_le32(mkdev_26(MAJOR(tty->device), MINOR(tty->device)))
-#else
-#    define IS_PTY_MASTER(tty) \
-      ((tty)->driver->major == UNIX98_PTY_MASTER_MAJOR || \
-      (tty)->driver->major == PTY_MASTER_MAJOR)
-#    define TTY_DEVNR(tty) \
-      cpu_to_le32(mkdev_26(tty->driver->major, \
-      tty->driver->minor_start + tty->index))
-#endif
+#define IS_PTY_MASTER(tty) \
+    ((tty)->driver->major == UNIX98_PTY_MASTER_MAJOR || \
+        (tty)->driver->major == PTY_MASTER_MAJOR)
+#define TTY_DEVNR(tty) \
+    cpu_to_le32(mkdev_26(tty->driver->major, \
+        tty->driver->minor_start + tty->index))
 #ifndef SEEK_SET
 #    define SEEK_SET 0
 #endif
@@ -74,7 +65,7 @@ kernel/rpldev.c - Kernel interface for RPLD
 #ifndef SEEK_END
 #    define SEEK_END 2
 #endif
-#define SKIP_PTM(tty) if(IS_PTY_MASTER(tty)) { return 0; }
+#define SKIP_PTM(tty)   if(IS_PTY_MASTER(tty)) return 0;
 
 // Stage 2 functions
 static int krpl_init(struct tty_struct *, struct tty_struct *, struct file *);
@@ -129,35 +120,26 @@ static struct miscdevice kmi_miscinfo = {
     .fops  = &kmi_fops,
 };
 
-extern __init int init_module(void);
-extern __exit void cleanup_module(void);
-MODULE_DESCRIPTION("ttyrpld Kernel interface");
+MODULE_DESCRIPTION("ttyrpld kernel interface");
 MODULE_AUTHOR("Jan Engelhardt <jengelh [at] linux01 gwdg de>");
 MODULE_LICENSE("GPL v2");
 MODULE_PARM_DESC(Bufsize, "Buffer size (default 32K)");
 MODULE_PARM_DESC(Minor_nr, "Minor number to use (default: 255(=DYNAMIC))");
-#ifdef KERNEL_2_4
-MODULE_PARM(Bufsize, "i");
-MODULE_PARM(Minor_nr, "i");
-EXPORT_NO_SYMBOLS;
-#else
 module_param(Bufsize, uint, S_IRUGO);
 module_param(Minor_nr, int, S_IRUGO);
-#endif
 
 //-----------------------------------------------------------------------------
-__init int init_module(void) {
+__init extern int init_module(void) {
     int eax;
     kmi_miscinfo.minor = Minor_nr;
-    if((eax = misc_register(&kmi_miscinfo)) != 0) {
+    if((eax = misc_register(&kmi_miscinfo)) != 0)
         return eax;
-    }
     Minor_nr = kmi_miscinfo.minor; // give minor back to sysfs
     printk(KERN_INFO PREFIX "registered at minor %d\n", Minor_nr);
     return 0;
 }
 
-__exit void cleanup_module(void) {
+__exit extern void cleanup_module(void) {
     misc_deregister(&kmi_miscinfo);
     return;
 }
@@ -173,17 +155,22 @@ static int krpl_init(struct tty_struct *tty, struct tty_struct *ctl,
     int len;
 
     SKIP_PTM(tty);
-    if(IS_ERR(full_dev = d_path(filp->f_dentry, filp->f_vfsmnt,
+
+    p.dev   = TTY_DEVNR(tty);
+    p.event = EVT_INIT;
+    p.magic = MAGIC_SIG;
+    fill_time(&p.time);
+
+    /* "/dev/tty" is an evil case, because its ownership is not the same as
+    that of a better node, e.g. /dev/tty1. Do not pass it to userspace. */
+    if(filp->f_dentry->d_inode->i_rdev == MKDEV(TTYAUX_MAJOR, 0) ||
+     IS_ERR(full_dev = d_path(filp->f_dentry, filp->f_vfsmnt,
      dev, sizeof(dev)))) {
         p.size = 0;
         return mv_buffer(&p, NULL, 0);
     }
 
-    p.size  = cpu_to_le16(len = strlen(full_dev));
-    p.dev   = TTY_DEVNR(tty);
-    p.event = EVT_INIT;
-    p.magic = MAGIC_SIG;
-    fill_time(&p.time);
+    p.size = cpu_to_le16(len = strlen(full_dev));
     return mv_buffer(&p, full_dev, len);
 }
 
@@ -197,17 +184,20 @@ static int krpl_open(struct tty_struct *tty, struct tty_struct *ctl,
     int len;
 
     SKIP_PTM(tty);
-    if(IS_ERR(full_dev = d_path(filp->f_dentry, filp->f_vfsmnt,
+
+    p.dev   = TTY_DEVNR(tty);
+    p.event = EVT_OPEN;
+    p.magic = MAGIC_SIG;
+    fill_time(&p.time);
+
+    if(filp->f_dentry->d_inode->i_rdev == MKDEV(TTYAUX_MAJOR, 0) ||
+     IS_ERR(full_dev = d_path(filp->f_dentry, filp->f_vfsmnt,
      dev, sizeof(dev)))) {
         p.size = 0;
         return mv_buffer(&p, NULL, 0);
     }
 
-    p.size  = cpu_to_le16(len = strlen(full_dev));
-    p.dev   = TTY_DEVNR(tty);
-    p.event = EVT_OPEN;
-    p.magic = MAGIC_SIG;
-    fill_time(&p.time);
+    p.size = cpu_to_le16(len = strlen(full_dev));
     return mv_buffer(&p, full_dev, len);
 }
 
@@ -221,7 +211,7 @@ static int krpl_read(const char __user *buf, size_t count,
     struct rpldev_packet p;
 
     SKIP_PTM(tty);
-    if(count == 0) { return 0; }
+    if(count == 0) return 0;
 
     p.dev   = TTY_DEVNR(tty);
     p.size  = cpu_to_le16(count);
@@ -252,7 +242,7 @@ static int krpl_write(const char __user *buf, size_t count,
     struct rpldev_packet p;
 
     SKIP_PTM(tty);
-    if(count == 0) { return 0; }
+    if(count == 0) return 0;
 
     p.dev   = TTY_DEVNR(tty);
     p.size  = cpu_to_le16(count);
@@ -297,7 +287,7 @@ static int krpl_close(struct tty_struct *tty, struct tty_struct *other) {
 static int krpl_deinit(struct tty_struct *tty, struct tty_struct *other) {
     struct rpldev_packet p;
 
-    if(IS_PTY_MASTER(tty)) { tty = other; }
+    if(IS_PTY_MASTER(tty)) tty = other;
 
     p.dev   = TTY_DEVNR(tty);
     p.size  = 0;
@@ -350,7 +340,8 @@ static int urpl_open(struct inode *inode, struct file *filp) {
       Inode Change Time: when the device is successfully opened
       Modification Time: whenever the device is opened
     */
-    if(inode != NULL) { inode->i_ctime = CURRENT_TIME; }
+    if(inode != NULL)
+        inode->i_ctime = CURRENT_TIME;
     return 0;
 }
 
@@ -360,17 +351,20 @@ static ssize_t urpl_read(struct file *filp, char __user *buf, size_t count,
     int eax;
 
     // Nothing read, nothing done
-    if(count == 0) { return 0; }
+    if(count == 0) return 0;
 
     // Must sleep as long as there is no data
-    if(down_interruptible(&Buffer_lock)) { return -ERESTARTSYS; }
+    if(down_interruptible(&Buffer_lock))
+        return -ERESTARTSYS;
+
     while(BufRP == BufWP) {
         up(&Buffer_lock);
-        if(filp->f_flags & O_NONBLOCK) { return -EAGAIN; }
-        if(wait_event_interruptible(Pull_queue, (BufRP != BufWP))) {
+        if(filp->f_flags & O_NONBLOCK)
+            return -EAGAIN;
+        if(wait_event_interruptible(Pull_queue, (BufRP != BufWP)))
+            return -ERESTARTSYS;
+        if(down_interruptible(&Buffer_lock))
             return -ERESTARTSYS;
-        }
-        if(down_interruptible(&Buffer_lock)) { return -ERESTARTSYS; }
     }
 
     // Data is available, so give it to the user
@@ -389,17 +383,19 @@ static loff_t urpl_seek(struct file *filp, loff_t offset, int origin) {
     with offset == 0. */
 
     if(origin == SEEK_END && offset == 0) {
-        if(down_interruptible(&Buffer_lock)) { return -ERESTARTSYS; }
+        if(down_interruptible(&Buffer_lock))
+            return -ERESTARTSYS;
         BufRP = BufWP;
         up(&Buffer_lock);
         filp->f_dentry->d_inode->i_atime = CURRENT_TIME;
         return 0;
     }
 
-    if(origin != SEEK_CUR) { return -ESPIPE; }
-    if(offset == 0) { return 0; }
+    if(origin != SEEK_CUR) return -ESPIPE;
+    if(offset == 0) return 0;
 
-    if(down_interruptible(&Buffer_lock)) { return -ERESTARTSYS; }
+    if(down_interruptible(&Buffer_lock))
+        return -ERESTARTSYS;
     BufRP = Buffer + (BufRP - Buffer + min_uint(offset, avail_R())) % Bufsize;
     if(BufRP >= Buffer + Bufsize) { 
         WARN_ON(1);
@@ -414,7 +410,9 @@ static loff_t urpl_seek(struct file *filp, loff_t offset, int origin) {
 static int urpl_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
  unsigned long arg)
 {
-    if(_IOC_TYPE(cmd) != RPL_IOC_MAGIC) { return -ENOTTY; /* ENAUGHTY */ }
+    if(_IOC_TYPE(cmd) != RPL_IOC_MAGIC)
+        return -ENOTTY; /* ENAUGHTY */
+
     switch(cmd) {
         case RPL_IOC_GETBUFSIZE:
             return Bufsize;
@@ -429,6 +427,7 @@ static int urpl_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
         case RPL_IOC_FLUSH:
             return urpl_seek(filp, 0, SEEK_END);
     }
+
     return -ENOTTY;
 }
 
@@ -456,13 +455,15 @@ static int urpl_close(struct inode *inode, struct file *filp) {
 //-----------------------------------------------------------------------------
 static inline ssize_t avail_R(void) {
     // Return the number of available bytes to read
-    if(BufWP >= BufRP) { return BufWP - BufRP; }
+    if(BufWP >= BufRP)
+        return BufWP - BufRP;
     return BufWP + Bufsize - BufRP;
 }
 
 static inline ssize_t avail_W(void) {
     // Return the number of available bytes to write
-    if(BufWP >= BufRP) { return BufRP + Bufsize - BufWP - 1; }
+    if(BufWP >= BufRP)
+        return BufRP + Bufsize - BufWP - 1;
     return BufRP - BufWP - 1;
 }
 
@@ -472,17 +473,16 @@ static inline void fill_time(struct timeval *tv) {
     /* The following code all gets optimized away on LE, yay! And even on BE,
     it is reduced by 50%. Just think of how ugly a #if-#endif wrappage
     would be. :-) */
-    if(sizeof(tv->tv_sec) == sizeof(uint32_t)) {
+    if(sizeof(tv->tv_sec) == sizeof(uint32_t))
         tv->tv_sec = cpu_to_le32(tv->tv_sec);
-    } else if(sizeof(tv->tv_sec) == sizeof(uint64_t)) {
+    else if(sizeof(tv->tv_sec) == sizeof(uint64_t))
         tv->tv_sec = cpu_to_le64(tv->tv_sec);
-    }
 
-    if(sizeof(tv->tv_usec) == sizeof(uint32_t)) {
+    if(sizeof(tv->tv_usec) == sizeof(uint32_t))
         tv->tv_usec = cpu_to_le32(tv->tv_usec);
-    } else if(sizeof(tv->tv_usec) == sizeof(uint64_t)) {
+    else if(sizeof(tv->tv_usec) == sizeof(uint64_t))
         tv->tv_usec = cpu_to_le64(tv->tv_usec);
-    }
+
     return;
 }
 
@@ -501,14 +501,16 @@ static int mv_buffer(struct rpldev_packet *p, const void *buf, size_t count) {
     /* Copy the contents the tty driver received to our circulary buffer and
     also add a header (rpld_packet) to it so that the userspace daemon can
     recognize the type. */
-    if(down_interruptible(&Buffer_lock)) { return -ERESTARTSYS; }
+    if(down_interruptible(&Buffer_lock))
+        return -ERESTARTSYS;
     if(avail_W() < sizeof(struct rpldev_packet) + count) {
         up(&Buffer_lock);
         return -ENOSPC;
     }
 
     mv_buffer2(p, sizeof(struct rpldev_packet));
-    if(count > 0) { mv_buffer2(buf, count); }
+    if(count > 0)
+        mv_buffer2(buf, count);
 
     up(&Buffer_lock);
     wake_up(&Pull_queue);
@@ -539,19 +541,17 @@ static inline int mv_to_user(char __user *dest, size_t count) {
     into userspace memory. The caller must make sure that COUNT is at most
     avail_R(). */
     size_t x = Buffer + Bufsize - BufRP;
-    int eax = 0;
+    int eax  = 0;
 
     /* Since this operation might block due to userspace page faults, the
     userspace daemon should really have DEST memory-locked. */
     if(count < x) {
-        if((eax = copy_to_user(dest, BufRP, count)) == 0) {
+        if((eax = copy_to_user(dest, BufRP, count)) == 0)
             BufRP += count;
-        }
     } else {
         if((eax = copy_to_user(dest, BufRP, x)) == 0 &&
-         (eax = copy_to_user(dest + x, Buffer, count - x)) == 0) {
+         (eax = copy_to_user(dest + x, Buffer, count - x)) == 0)
             BufRP = Buffer + count - x;
-        }
     }
 
     return eax;
