@@ -37,7 +37,7 @@ user/infod.c - INFO socket for RPLD
 #include <libHX.h>
 #include "rpl_stdint.h"
 #include "dev.h"
-#include "global.h"
+#include "lib.h"
 #include "rdsh.h"
 
 // Function prototypes
@@ -82,9 +82,9 @@ void *infod_main(void *arg) {
         pthread_t id;
         int clfd;
 
-        if((clfd = accept(Svfd, (struct sockaddr *)&remote, &sz)) < 0) {
+        if((clfd = accept(Svfd, (struct sockaddr *)&remote, &sz)) < 0)
             continue;
-        }
+
         pthread_create(&id, NULL, client_thread, (void *)clfd);
         pthread_detach(id);
     }
@@ -103,8 +103,9 @@ static void *client_thread(void *arg) {
 
         eax |= !RECEIVE_FULL(fd, &req, sizeof(unsigned char));
         eax |= !RECEIVE_FULL(fd, &gint, sizeof(gint));
-        SWAB(&gint, sizeof(gint));
-        if(eax || req == IFP_NONE) { break; }
+        SWAB1(&gint);
+        if(eax || req == IFP_NONE)
+            break;
 
         switch(req) {
             case IFP_ACTIVATE ... IFP_DEACTIVSES:
@@ -113,7 +114,8 @@ static void *client_thread(void *arg) {
             case IFP_REMOVE: {
                 struct tty *tty;
                 pthread_mutex_lock(&Ttys_lock);
-                if((tty = get_tty(gint, 0)) != NULL) { log_close(tty); }
+                if((tty = get_tty(gint, 0)) != NULL)
+                    log_close(tty);
                 pthread_mutex_unlock(&Ttys_lock);
                 break;
             }
@@ -161,9 +163,8 @@ static void getinfo_text(uint32_t dev, int fd) {
 
         travp = HXbtrav_init(Ttys, NULL);
 
-        while((nd = HXbtraverse(travp)) != NULL) {
+        while((nd = HXbtraverse(travp)) != NULL)
             print_entry_text(fd, nd->data);
-        }
 
         skprintf(fd, "-----------------------------------------------"
          "---------------------------\n");
@@ -196,9 +197,8 @@ static void getinfo(uint32_t dev, int fd) {
 
         travp = HXbtrav_init(Ttys, NULL);
 
-        while((nd = HXbtraverse(travp)) != NULL) {
+        while((nd = HXbtraverse(travp)) != NULL)
             print_entry(fd, nd->data);
-        }
 
         HXbtrav_free(travp);
     } else {
@@ -259,9 +259,8 @@ static inline const char *basename_pp(const char *s, const char *t) {
     if(s == NULL) { return NULL; }
 
     if((p = strchr(t, '%')) != NULL) {
-        if((p = HX_strbchr(t, p, '/')) != NULL) {
+        if((p = HX_strbchr(t, p, '/')) != NULL)
             return s + (p - t) + 1;
-        }
         return s;
     }
 
@@ -308,7 +307,7 @@ static int skprintf(int fd, const char *fmt, ...) {
 
     va_start(argp, fmt);
     le = s = vsnprintf(buf, sizeof(buf), fmt, argp);
-    SWAB(&le, sizeof(size_t));
+    SWAB1(&le);
     send_wait(fd, &le, sizeof(uint32_t), 0);
     send_wait(fd, buf, s, 0);
 
@@ -327,9 +326,8 @@ static int unix_server(const char *port) {
 
     if((fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0 ||
      bind(fd, (struct sockaddr *)&sk, sizeof(struct sockaddr_un)) < 0 ||
-     listen(fd, SOMAXCONN) < 0) {
+     listen(fd, SOMAXCONN) < 0)
         return -1;
-    }
 
     return fd;
 }
