@@ -26,7 +26,6 @@ user/replay.c - Realtime TTY log analyzer
 #include <sys/types.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <sched.h> // sched_yield()
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -36,7 +35,7 @@ user/replay.c - Realtime TTY log analyzer
 #include <libHX.h>
 #include "dev.h"
 #include "rpl_packet.h"
-#include "global.h"
+#include "lib.h"
 #include "pctrl.h"
 
 #define offsetof(type, member) ((size_t)&((type *)NULL)->member)
@@ -74,15 +73,15 @@ static void e_proc(int, struct rpldsk_packet *, struct pctrl_info *, char *,
   struct timeval *, long *);
 
 static unsigned long calc_ovcorr(unsigned long, int);
-static int find_next_packet(int);
+static int find_next_packet(int, const struct pctrl_info *);
 static int get_options(int *, const char ***);
 static void getopt_jump(const struct HXoptcb *);
 static void getopt_msec(const struct HXoptcb *);
 static void getopt_skip(const struct HXoptcb *);
 static inline ssize_t read_nullfm(int, size_t);
 static ssize_t read_through(int, int, size_t);
-static ssize_t read_waitfm(int, void *, size_t);
-static int seek_to_end(int);
+static ssize_t read_waitfm(int, void *, size_t, const struct pctrl_info *);
+static int seek_to_end(int, const struct pctrl_info *);
 static int usleep_ovcorr(struct timeval *, long *);
 static void tv_delta(const struct timeval *, const struct timeval *,
  struct timeval *);
@@ -95,7 +94,8 @@ int main(int argc, const char **argv) {
     int i;
 
     load_locale(*argv);
-    if(get_options(&argc, &argv) <= 0) { return EXIT_FAILURE; }
+    if(get_options(&argc, &argv) <= 0)
+        return EXIT_FAILURE;
 
     fprintf(stderr, "> ttyreplay " TTYRPLD_VERSION "\n");
     fprintf(stderr,
@@ -105,6 +105,8 @@ int main(int argc, const char **argv) {
       " have come with this program.\n")
     );
 
+    ++argv;
+    --argc;
     if(argc == 0) {
         if(!isatty(STDIN_FILENO)) {
             fprintf(stderr, _("No arguments given but STDIN seems to be a "
@@ -118,7 +120,8 @@ int main(int argc, const char **argv) {
 
     setvbuf(stdout, NULL, _IONBF, 0);
     Opt.ovcorr = calc_ovcorr(0, 100);
-    if(!Opt.no_pctrl) { pctrl_init(); }
+    if(!Opt.no_pctrl)
+        pctrl_init();
 
     for(i = 0; i < argc; /* see switch() */) {
         const char *file = (argv[i][0] == '-' && argv[i][1] == '\0') ?
@@ -135,10 +138,10 @@ int main(int argc, const char **argv) {
 
         switch(rv) {
             case PCTRL_PREV:
-                if(i > 0) { --i; }
+                if(i > 0) --i;
                 break;
             case PCTRL_NEXT:
-                if(i < argc - 1) { ++i; }
+                if(i < argc - 1) ++i;
                 break;
             case PCTRL_EXIT:
                 i = argc;
@@ -161,7 +164,8 @@ static int replay_file(int fd, const char *name) {
     long skew = 0;
     char tick = 0;
 
-    if(Opt.follow == FM_LIVE && !seek_to_end(fd)) { return 0; }
+    if(Opt.follow == FM_LIVE && !seek_to_end(fd, &ps))
+        return 0;
 
     memset(&ps, 0, sizeof(struct pctrl_info));
     ps.echo   = 0;
@@ -170,21 +174,21 @@ static int replay_file(int fd, const char *name) {
     ps.skval  = Opt.skval;
     pctrl_activate(&ps);
 
-    while((eax = read_waitfm(fd, &packet, sizeof(struct rpldsk_packet))) ==
-     sizeof(struct rpldsk_packet)) {
+    while((eax = read_waitfm(fd, &packet,
+     sizeof(struct rpldsk_packet), &ps)) == sizeof(struct rpldsk_packet)) {
         if(packet.magic != MAGIC_SIG) {
             fprintf(stderr, _("\n" "<Packet inconsistency! "
              "Trying to find next valid packet.>\n"));
             tick = 0;
             /* If read() in find_next_packet() generates an error, it will be
             catched when the condition is re-evaulated upon continue. */
-            find_next_packet(fd);
+            find_next_packet(fd, &ps);
             continue;
         }
 
-        SWAB(&packet.size, sizeof(packet.size));
-        SWAB(&packet.tv.tv_sec, sizeof(packet.tv.tv_sec));
-        SWAB(&packet.tv.tv_usec, sizeof(packet.tv.tv_usec));
+        SWAB1(&packet.size);
+        SWAB1(&packet.tv.tv_sec);
+        SWAB1(&packet.tv.tv_usec);
 
         switch(packet.event) {
             case EVT_WRITE:
@@ -200,10 +204,19 @@ static int replay_file(int fd, const char *name) {
                 read_through(fd, STDERR_FILENO, packet.size);
                 fprintf(stderr, "\e[0m\n"); // ]
                 break;
+            case EVT_ID_TIME:
+                fprintf(stderr, "\e[1;37;41m%s ", _("Recorded on")); // ]
+                read_through(fd, STDERR_FILENO, packet.size);
+                fprintf(stderr, "\e[0m\n"); // ]
+                break;
+            case EVT_ID_USER:
+                fprintf(stderr, "\e[1;37;41m%s: ", _("User")); // ]
+                read_through(fd, STDERR_FILENO, packet.size);
+                fprintf(stderr, "\e[0m\n"); // ]
+                break;
             case EVT_DEINIT:
                 fprintf(stderr, "\n\e[1;37;42m%s\e[0m\n", // ]]
                  _("<tty device has been closed>"));
-                // fallthrough kinda
                 read_nullfm(fd, packet.size);
                 break;
             case EVT_READ:
@@ -213,21 +226,22 @@ static int replay_file(int fd, const char *name) {
                     printf("\e[0m"); // ]
                     break;
                 }
-                // true fallthrough
+                // fallthrough
             default:
+                /* default case: ignore unknown (may be new) packet types
+                this version does not support (yet). */
                 read_nullfm(fd, packet.size);
                 break;
         }
 
-        if(ps.brk) { break; }
+        if(ps.brk) break;
     }
 
     pctrl_deactivate(0);
 
-    if(eax < 0) {
+    if(eax < 0)
         fprintf(stderr, "\n\e[1;37;41m%s\e[0m\nread(): %s\n", // ]]
          _("<Error while reading from stream>"), strerror(errno));
-    }
 
     printf("\n\e[1;37;41m"); // ]
     printf(_("<Log replaying of %s finished>"), name);
@@ -254,7 +268,7 @@ static void e_proc(int fd, struct rpldsk_packet *p, struct pctrl_info *i,
         }
     } else if(Opt.follow != FM_LIVE && Opt.follow != FM_NODELAY) {
         /* ... only when we know the next packet (which is already read by now)
-        the tiem difference can be calculated. This code is not executed in
+        the time difference can be calculated. This code is not executed in
         FM_LIVE mode, because we have been already waiting enough with
         read_wait(). */
         struct timeval delta;
@@ -266,18 +280,20 @@ static void e_proc(int fd, struct rpldsk_packet *p, struct pctrl_info *i,
             read_through(fd, STDOUT_FILENO, p->size);
             return;
         }
-        if(i->factor != 1.0) {
+
+        if(i->factor != 1.0)
             // -S switch for the fast
             usec2tv((double)tv2usec(&delta) / i->factor, &delta);
-        }
-        if(Opt.maxdelay > 0 && tv2usec(&delta) > Opt.maxdelay) {
+
+        if(Opt.maxdelay > 0 && tv2usec(&delta) > Opt.maxdelay)
             // -m switch for those who do not want to wait their lifetime.
             usec2tv(Opt.maxdelay, &delta);
-        }
+
         usleep_ovcorr(&delta, skew);
     }
 
-    while(i->paused) { sched_yield(); }
+    while(i->paused)
+        usleep(100000);
 
     memcpy(stamp, &p->time, sizeof(struct timeval));
     read_through(fd, STDOUT_FILENO, p->size);
@@ -287,13 +303,14 @@ static void e_proc(int fd, struct rpldsk_packet *p, struct pctrl_info *i,
         char buf[80];
         time_t now = p->time.tv_sec;
         localtime_r(&now, &tm);
-        if(Opt.showtime == 1) {
+
+        if(Opt.showtime == 1)
             strftime(buf, sizeof(buf), "\e7\e[1;300H\e[9D" // ]]
-             "\e[0;7m[\e[0;1;37;41m" "%H:%M:%S" "\e[0;7m]\e8", &tm); // ]]]
-        } else {
+             "\e[0;7m[\e[0;1;37;41m" "%H:%M:%S" "\e[0;7m]\e8\e[0m", &tm); // ]]]]
+        else
             strftime(buf, sizeof(buf), "\e7\e[1;300H\e[12D" // ]]
-             "\e[0;7m[\e[0;1;37;41m" "%d.%m %H:%M" "\e[0;7m]\e8", &tm); // ]]]
-        }
+             "\e[0;7m[\e[0;1;37;41m" "%d.%m %H:%M" "\e[0;7m]\e8\e[0m", &tm); // ]]]]
+
         write(STDOUT_FILENO, buf, strlen(buf));
     }
     return;
@@ -321,9 +338,9 @@ static unsigned long calc_ovcorr(unsigned long ad, int rd) {
     return av;
 }
 
-static int find_next_packet(int fd) {
-    /* There is more algorithmic code than simple code in ttyreplay.
-    Outstanding. */
+static int find_next_packet(int fd, const struct pctrl_info *ps) {
+    /* Interesting, there is more algorithmic code than simple
+    code in ttyreplay. Outstanding. */
 #define LZ           sizeof(struct rpldsk_packet)
 #define BZ           (2 * LZ)
 #define MAGIC_OFFSET offsetof(struct rpldsk_packet, magic)
@@ -332,7 +349,8 @@ static int find_next_packet(int fd) {
     size_t s;
     int ok = 0;
 
-    if(read_waitfm(fd, buf, BZ) < BZ) { return 0; }
+    if(read_waitfm(fd, buf, BZ, ps) < BZ)
+        return 0;
 
     while(1) {
         char *ptr;
@@ -348,22 +366,23 @@ static int find_next_packet(int fd) {
             if(ctx != buf) {
                 size_t cnt = buf + BZ - ctx;
                 ctx = memmove(buf, ctx, cnt);
-                if(read_waitfm(fd, buf + cnt, BZ - cnt) < BZ - cnt) {
+                if(read_waitfm(fd, buf + cnt, BZ - cnt, ps) < BZ - cnt)
                     return 0;
-                }
             }
         } else if(ptr != NULL) {
             /* Magic byte, but of no use. Discard it, and fill up with new
             data from the descriptor. */
             size_t cnt = buf + BZ - ptr - 1;
             memmove(buf, ptr + 1, cnt);
-            if(read_waitfm(fd, buf + cnt, BZ - cnt) < BZ - cnt) { return 0; }
+            if(read_waitfm(fd, buf + cnt, BZ - cnt, ps) < BZ - cnt)
+                return 0;
             continue;
         } else {
             /* No magic byte, but since it might be just the next byte in
             the stream, only read LZ bytes. */
             memmove(buf, buf + BZ - LZ, BZ - LZ);
-            if(read_waitfm(fd, buf + LZ, BZ - LZ) < BZ - LZ) { return 0; }
+            if(read_waitfm(fd, buf + LZ, BZ - LZ, ps) < BZ - LZ)
+                return 0;
             continue;
         }
 
@@ -373,21 +392,20 @@ static int find_next_packet(int fd) {
             questionable at all. Start with a new slate. */
             ok = 0;
             memmove(buf, buf + LZ, BZ - LZ);
-            if(read_waitfm(fd, buf, BZ) < BZ) { return 0; }
+            if(read_waitfm(fd, buf, BZ, ps) < BZ)
+                return 0;
             continue;
         }
 
         if(s < LZ) {
             memmove(buf, buf + LZ + s, BZ - LZ - s);
-            if(read_waitfm(fd, buf + BZ - LZ - s, LZ + s) < LZ + s) {
+            if(read_waitfm(fd, buf + BZ - LZ - s, LZ + s, ps) < LZ + s)
                 return 0;
-            }
         } else {
             /* There is no header (according to .size) in our buffer, so we
             can blindly munge lots of data. */
-            if(!read_nullfm(fd, s - LZ) || read_waitfm(fd, buf, BZ) < BZ) {
+            if(!read_nullfm(fd, s - LZ) || read_waitfm(fd, buf, BZ, ps) < BZ)
                 return 0;
-            }
         }
 
         if((ptr = memchr(buf, MAGIC_SIG, MAGIC_OFFSET + 1)) == NULL ||
@@ -398,7 +416,7 @@ static int find_next_packet(int fd) {
             continue;
         }
 
-        if(++ok >= 2) { break; }
+        if(++ok >= 2) break;
     }
 
     fprintf(stderr, _("\n" "<Found packet boundary>\n"));
@@ -408,13 +426,13 @@ static int find_next_packet(int fd) {
         /* Mmhkay, there is another header other than the current (packet)
         in the buffer. Crap, another one gone. */
         memmove(buf, buf + LZ + s, BZ - LZ - s);
-        if(read_waitfm(fd, buf + BZ - LZ - s, s) < s ||
-         !read_nullfm(fd, packet->size)) {
+        if(read_waitfm(fd, buf + BZ - LZ - s, s, ps) < s ||
+         !read_nullfm(fd, packet->size))
             return 0;
-        }
     } else {
         // Just subtract what we have already read into the buffer
-        if(!read_nullfm(fd, s - LZ)) { return 0; }
+        if(!read_nullfm(fd, s - LZ))
+            return 0;
     }
 
     return 1;
@@ -425,24 +443,24 @@ static int find_next_packet(int fd) {
 
 static int get_options(int *argc, const char ***argv) {
     struct HXoption options_table[] = {
-        {.ln = "no-pctrl", .type = HXOPT_NONE, .ptr = &Opt.no_pctrl,
+        {.ln = "no-pctrl", .type = HXTYPE_NONE, .ptr = &Opt.no_pctrl,
          .help = _("Do not enable playing controls")},
-        {.sh = 'F', .type = HXOPT_VAL, .ptr = &Opt.follow, .val = FM_LIVE,
+        {.sh = 'F', .type = HXTYPE_VAL, .ptr = &Opt.follow, .val = FM_LIVE,
          .help = _("Live feed follow mode (like `tail -f`)")},
-        {.sh = 'J', .type = HXOPT_STRING, .cb = getopt_jump,
+        {.sh = 'J', .type = HXTYPE_STRING, .cb = getopt_jump,
          .help = _("Seek to time position (HH:MM:SS or total SEC, "
          "see docs)"), .htyp = "tspec"},
-        {.sh = 'S', .type = HXOPT_DOUBLE, .ptr = &Opt.factor,
+        {.sh = 'S', .type = HXTYPE_DOUBLE, .ptr = &Opt.factor,
          .help = _("Speed factor (default: 1.0)")},
-        {.sh = 'T', .type = HXOPT_VAL, .ptr = &Opt.follow, .val = FM_NODELAY,
+        {.sh = 'T', .type = HXTYPE_VAL, .ptr = &Opt.follow, .val = FM_NODELAY,
          .help = _("No delays (useful for text file grepping)")},
-        {.sh = 'f', .type = HXOPT_VAL, .ptr = &Opt.follow, .val = FM_CATCHUP,
+        {.sh = 'f', .type = HXTYPE_VAL, .ptr = &Opt.follow, .val = FM_CATCHUP,
          .help = _("Catch-up follow mode (play file, switch to live feed on EOF)")},
-        {.sh = 'j', .type = HXOPT_LONG, .ptr = &Opt.skval, .cb = getopt_skip,
+        {.sh = 'j', .type = HXTYPE_LONG, .ptr = &Opt.skval, .cb = getopt_skip,
          .help = _("Skip the given number of EVT_WRITE packets")},
-        {.sh = 'm', .type = HXOPT_LONG, .cb = getopt_msec,
+        {.sh = 'm', .type = HXTYPE_LONG, .cb = getopt_msec,
          .help = _("Maximum delay to wait between packets"), .htyp = _("msec")},
-        {.sh = 't', .type = HXOPT_NONE | HXOPT_INC, .ptr = &Opt.showtime,
+        {.sh = 't', .type = HXTYPE_NONE | HXOPT_INC, .ptr = &Opt.showtime,
          .help = _("Show the timestamp at the top of the screen")},
         HXOPT_AUTOHELP,
         HXOPT_TABLEEND,
@@ -493,11 +511,13 @@ static ssize_t read_through(int in, int out, size_t count) {
 
     while(rem > 0) {
         ssize_t eax = read(in, buf, min_uint(BUFSIZE, rem));
-        if(eax < 0) { return 0; }
+        if(eax < 0)
+            return 0;
         write(out, buf, eax);
-        if(eax == rem) { break; }
+        if(eax == rem)
+            break;
         rem -= eax;
-        sched_yield();
+        usleep(10000);
     }
 
     return count;
@@ -508,19 +528,20 @@ static inline ssize_t read_nullfm(int fd, size_t count) {
      Opt.follow != FM_NODELAY);
 }
 
-static ssize_t read_waitfm(int fd, void *buf, size_t count) {
+static ssize_t read_waitfm(int fd, void *buf, size_t count,
+ const struct pctrl_info *ps)
+{
     /* Wrapper function for either read() or read_wait(). If EOF has been
     detected (ATM), we switch into FM_LIVE mode (if currently in FM_CATCHUP
     mode). */
     struct stat sb;
     off_t pos;
 
-    if(Opt.follow == FM_NONE || Opt.follow == FM_NODELAY) {
+    if(Opt.follow == FM_NONE || Opt.follow == FM_NODELAY)
         /* If no follow mode is selected, the while() loop in replay_file()
         shall terminate as soon as we encounter EOF or have a short read (i.e.
         read less then count). */
         return read(fd, buf, count);
-    }
 
     /* If it is a live feed (follow mode), complete the read (i.e. read all
     requested bytes. fstat() must come after lseek() -- in the very case that
@@ -532,10 +553,10 @@ static ssize_t read_waitfm(int fd, void *buf, size_t count) {
         Opt.follow = FM_LIVE;
     }
 
-    return read_wait(fd, buf, count);
+    return read_wait(fd, buf, count, ps);
 }
 
-static int seek_to_end(int fd) {
+static int seek_to_end(int fd, const struct pctrl_info *ps) {
     if(lseek(fd, 0, SEEK_END) == -1 && errno == ESPIPE) {
         /* Some workaround for non-seekable descriptors. G_skip() is not
         suitable, since ... it's just not designed to do what is below, i.e.
@@ -556,25 +577,26 @@ static int seek_to_end(int fd) {
         fcntl(fd, F_SETFL, fcntl(fd, F_GETFL) & ~O_NONBLOCK);
     }
 
-    return find_next_packet(fd);
+    return find_next_packet(fd, ps);
 }
 
 static int usleep_ovcorr(struct timeval *req, long *skew) {
-    /* The manual page for nanosleep(2) says this under the "BUGS" section:
+    /* The manual page for nanosleep(2) says this under the "BUGS" section
+    [this copy adjusted by Jan Engelhardt for current events]:
 
       >The current implementation of nanosleep is  based  on  the
       >normal  kernel  timer mechanism, which has a resolution of
-      >1/HZ s (i.e, 10 ms on Linux/i386 and 1 ms on Linux/Alpha).
+      >1/HZ s.
 
-    (*) for i386/2.4: HZ=100, i.e. 10 ms (10000 µs)
-        for  all/2.6: HZ=1000, i.e. 1 ms (1000 µs)
+    (This may vary, as one can set HZ in the kernel menuconfig.
+    1024 HZ is/was used instead of 1000 HZ for some arches like alpha.)
 
-    I do not claim that all 2.4 have HZ=100, especially since distributors
-    like to change such. Just as likely, someone may set HZ back to 100 for
-    a 2.6 one.
+        100 HZ = 10 ms = 10000 µs
+        250 HZ =  4 ms =  4000 µs
+       1000 HZ =  1 ms =  1000 µs
 
       >Therefore, nanosleep pauses always for at least the speci-
-      >fied time, however it can take up to  10  ms  longer  than
+      >fied time, however it can take up to  4  ms  longer  than
       >specified  until  the  process becomes runnable again. [...]
 
       >As some applications  require  much  more  precise  pauses
@@ -590,8 +612,8 @@ static int usleep_ovcorr(struct timeval *req, long *skew) {
 
     Thus, I have designed a function that tries to cope with the longer delays
     by waiting a little shorter the next time when the previous call took
-    longer. The minimal margin of course still is 1/100 (1/1000) s, why the
-    algorithm splits up into multiple cases to make the best performance. They
+    longer. The minimal margin of course still is 1/250 s, why the algorithm
+    splits up into multiple cases to make the best performance. They
     are explained below. */
 
     struct timeval start, stop;
