@@ -34,7 +34,7 @@ user/rplctl.c - INFOD client
 
 #include <libHX.h>
 #include "dev.h"
-#include "global.h"
+#include "lib.h"
 #include "rdsh.h"
 
 #define IFP_GETINFO_A (Opt.parseable ? IFP_GETINFO : IFP_GETINFO_T)
@@ -64,15 +64,13 @@ int rplctl_main(int argc, const char **argv) {
         return EXIT_FAILURE;
     }
 
-    if(get_options(&argc, &argv) <= 0) {
+    if(get_options(&argc, &argv) <= 0)
         return EXIT_FAILURE;
-    }
 
     if(Got_arg == 0) {
-        if(!Opt.parseable) {
+        if(!Opt.parseable)
             printf("A %-7s %13s %9s %s\n", _("TTY"), _("BYTES IN"),
              _("OUT"), _("FILENAME"));
-        }
         send_int(Sockfd, IFP_GETINFO_A, 0);
     }
 
@@ -86,7 +84,7 @@ static uint32_t getdev(const char *s) {
     uint32_t major, minor;
     int found = 0;
 
-    if(s == NULL) { return 0; }
+    if(s == NULL) return 0;
     if(*s == '=') {
         /* The user can specify a major-minor number combination
         in form of "=MAJOR,MINOR". */
@@ -142,8 +140,9 @@ static void read_reply(int fd) {
         fd_set rd;
 
         recv(fd, &reply_size, sizeof(uint32_t), MSG_WAITALL);
-        SWAB(&reply_size, sizeof(uint32_t));
-        if(reply_size == 0) { break; }
+        SWAB1(&reply_size);
+        if(reply_size == 0)
+            break;
 
         while(reply_size > 0) {
             FD_ZERO(&rd);
@@ -162,7 +161,7 @@ static void read_reply(int fd) {
 
 static void send_int(int fd, unsigned char req, uint32_t dev) {
     send_wait(fd, &req, sizeof(unsigned char), 0);
-    SWAB(&dev, sizeof(uint32_t));
+    SWAB1(&dev);
     send_wait(fd, &dev, sizeof(uint32_t), 0);
     read_reply(fd);
     return;
@@ -177,9 +176,8 @@ static int unix_client(const char *port) {
     sk.sun_path[sizeof(sk.sun_path) - 1] = '\0';
 
     if((fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0 ||
-     connect(fd, (struct sockaddr *)&sk, sizeof(struct sockaddr_un)) < 0) {
+     connect(fd, (struct sockaddr *)&sk, sizeof(struct sockaddr_un)) < 0)
         return -1;
-    }
 
     return fd;
 }
@@ -187,22 +185,22 @@ static int unix_client(const char *port) {
 //-----------------------------------------------------------------------------
 static int get_options(int *argc, const char ***argv) {
     struct HXoption options_table[] = {
-        {.sh = 'A', HXOPT_STRING, .cb = getopt_proc,
+        {.sh = 'A', HXTYPE_STRING, .cb = getopt_proc,
          .help = _("Capture data from tty"), _("tty")},
-        {.sh = 'D', HXOPT_STRING, .cb = getopt_proc,
+        {.sh = 'D', HXTYPE_STRING, .cb = getopt_proc,
          .help = _("Ignore data from tty"), _("tty")},
-        {.sh = 'L', HXOPT_STRING | HXOPT_OPTIONAL, .cb = getopt_proc,
+        {.sh = 'L', HXTYPE_STRING | HXOPT_OPTIONAL, .cb = getopt_proc,
          .help = _("Explicitly request info about all [or given] ttys"),
          .htyp = _("tty")},
-        {.sh = 'S', HXOPT_STRING, .cb = getopt_proc,
+        {.sh = 'S', HXTYPE_STRING, .cb = getopt_proc,
          .help = _("Ignore data from tty until next tty session"), _("tty")},
-        {.sh = 'X', HXOPT_STRING, .cb = getopt_proc,
+        {.sh = 'X', HXTYPE_STRING, .cb = getopt_proc,
          .help = _("Close the tty's logfile"), _("tty")},
-        {.sh = 'Z', HXOPT_STRING | HXOPT_OPTIONAL, .cb = getopt_proc,
+        {.sh = 'Z', HXTYPE_STRING | HXOPT_OPTIONAL, .cb = getopt_proc,
          .help = _("Zero all packet and byte counters"), .htyp = _("tty")},
-        {.sh = 'f', HXOPT_STRING, .ptr = &GOpt.infod_port,
+        {.sh = 'f', HXTYPE_STRING, .ptr = &GOpt.infod_port,
          .help = _("Path to the INFOD socket"), .htyp = _("file")},
-        {.sh = 't', HXOPT_NONE, .ptr = &Opt.parseable,
+        {.sh = 't', HXTYPE_NONE, .ptr = &Opt.parseable,
          .help = _("Generate grepable output")},
         HXOPT_AUTOHELP,
         HXOPT_TABLEEND,
@@ -226,9 +224,8 @@ static void getopt_proc(const struct HXoptcb *cbi) {
         case 'X': {
             uint32_t dev;
             ++Got_arg;
-            if((dev = getdev(cbi->s)) != 0) {
+            if((dev = getdev(cbi->s)) != 0)
                 send_int(Sockfd, mapping[(int)cbi->tsh], dev);
-            }
             break;
         }
         case 'L': {
@@ -246,4 +243,5 @@ static void getopt_proc(const struct HXoptcb *cbi) {
     }
     return;
 }
+
 //=============================================================================
