@@ -2,6 +2,7 @@
 ttyrpld - TTY replay daemon
 user/rpld.c - User space daemon (filtering, etc.)
   Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004 - 2005
+  Copyright © Nguyen Anh Quynh <aquynh [at] gmail com>, 2005
   -- License restrictions apply (GPL v2)
 
   This file is part of ttyrpld.
@@ -45,18 +46,15 @@ extern int posix_memalign(void **, size_t, size_t);
 #include "rpl_endian.h"
 #include "rpl_ioctl.h"
 #include "rpl_packet.h"
-#include "global.h"
+#include "lib.h"
 #include "rdsh.h"
 
-#ifndef S_IRUGO
-#    define S_IRUGO (S_IRUSR | S_IRGRP | S_IROTH)
-#endif
-#ifndef S_IWUGO
-#    define S_IWUGO (S_IWUSR | S_IWGRP | S_IWOTH)
-#endif
-#ifndef S_IXUGO
-#    define S_IXUGO (S_IXUSR | S_IXGRP | S_IXOTH)
-#endif
+enum {
+    C_LOGOPEN = 0,
+    C_BADPACKET,
+    C_PKTTYPE,
+    C_MAX,
+};
 
 // Overview: The prototype order is the same as the function body order
 static void mainloop(int);
@@ -79,17 +77,16 @@ static void sighandler_int(int);
 static void sighandler_alrm(int);
 static void sighandler_pipe(int);
 
-static int get_options(int *, const char ***);
-static void getopt_config(const struct HXoptcb *);
-static void getopt_username(const struct HXoptcb *);
-static int read_config(const char *);
-static int read_config_bp(const char *, const char *);
-static int rdconfig_user(const struct shconf_opt *, const char *);
-
 static int find_devnode(uint32_t, char *, size_t, const char **);
 static int find_devnode_dive(uint32_t, char *, size_t, const char *);
 static char *getnamefromuid(uid_t, char *, size_t);
 static uid_t getuidfromname(const char *);
+static int get_options(int *, const char ***);
+static void getopt_config(const struct HXoptcb *);
+static void getopt_username(const struct HXoptcb *);
+static int rate_limit(int, time_t);
+static int read_config(const char *);
+static int read_config_bp(const char *, const char *);
 
 // Global variables (begin with uppercase character)
 static void *Buffer = NULL;
@@ -113,24 +110,25 @@ int main(int argc, const char **argv) {
     int fd;
 
     load_locale(*argv);
-    umask(umask(S_IWUSR | S_IWOTH) | S_IRUGO | S_IRWXG | S_IRWXO); // 0022,0477
+    umask(~(S_IWUSR | S_IXUSR));
 
     /* Yep, the config file is what is needed by all three
     (rpld, infod, rplctl). */
-    if(!read_config("/etc/rpld.conf") && errno != ENOENT) {
+    if(!read_config("/etc/rpld.conf") && errno != ENOENT)
         fprintf(stderr, _("/etc/rpld.conf exists but could not be read: %s\n"),
          strerror(errno));
-    }
-    if(!read_config_bp(*argv, "rpld.conf") && errno != ENOENT) {
+    if(!read_config("/usr/local/etc/rpld.conf") && errno != ENOENT)
+        fprintf(stderr, _("/usr/local/etc/rpld.conf exists but could not be read: %s\n"),
+         strerror(errno));
+    if(!read_config_bp(*argv, "rpld.conf") && errno != ENOENT)
         fprintf(stderr, _("$BINPATH/rpld.conf exists but could not be"
          " read: %s\n"), strerror(errno));
-    }
 
-    if(strcmp(HX_basename(*argv), "rplctl") == 0) {
+    if(strcmp(HX_basename(*argv), "rplctl") == 0)
         return rplctl_main(argc, argv);
-    }
 
-    if(get_options(&argc, &argv) <= 0) { return EXIT_FAILURE; }
+    if(get_options(&argc, &argv) <= 0)
+        return EXIT_FAILURE;
     memset(&Stats, 0, sizeof(Stats));
 
     if(GOpt.verbose) {
@@ -152,14 +150,14 @@ int main(int argc, const char **argv) {
         return EXIT_FAILURE;
     }
 
-    if(init_sbuffer(fd) < 0) { return EXIT_FAILURE; }
+    if(init_sbuffer(fd) < 0) return EXIT_FAILURE;
     init_sighandler();
 
-    if(Opt.infod_start)  { infod_init(); }
-    if(GOpt.syslog)      { openlog("rpld", LOG_PID, LOG_DAEMON); }
-    if(GOpt.user_id > 0) { setuid(GOpt.user_id); }
-    if(Opt.infod_start)  { pthread_create(&infod_id, NULL, infod_main, NULL); }
-    if(GOpt.verbose)     { alarm(1); }
+    if(Opt.infod_start)  infod_init();
+    if(GOpt.syslog)      openlog("rpld", LOG_PID, LOG_DAEMON);
+    if(GOpt.user_id > 0) setuid(GOpt.user_id);
+    if(Opt.infod_start)  pthread_create(&infod_id, NULL, infod_main, NULL);
+    if(GOpt.verbose)     alarm(1);
     mainloop(fd);
 
     if(Opt.infod_start) {
@@ -188,24 +186,25 @@ static void mainloop(int fd) {
             fstat(fd, &sb);
             if(!S_ISREG(sb.st_mode)) {
 #ifdef __OpenBSD__
-                if(e == EINTR) { continue; }
+                if(e == EINTR) continue;
 #endif
-                fprintf(stderr, _("\n" "Short read: %ld bytes only. Error %d:"
-                 " %s\n"), (long)eax, errno, strerror(errno));
+                fprintf(stderr, _("\n" "Short read: %ld bytes only. "
+                 "Error %d: %s\n"), (long)eax, errno, strerror(errno));
             }
-            Opt._running = 0;
+//            Opt._running = 0;
             break;
         }
 
-        SWAB(&packet.dev, sizeof(packet.dev));
-        SWAB(&packet.size, sizeof(packet.size));
-        SWAB(&packet.time.tv_sec, sizeof(packet.time.tv_sec));
-        SWAB(&packet.time.tv_usec, sizeof(packet.time.tv_usec));
+        SWAB1(&packet.dev);
+        SWAB1(&packet.size);
+        SWAB1(&packet.time.tv_sec);
+        SWAB1(&packet.time.tv_usec);
 
         if(packet.magic != MAGIC_SIG) {
             ++Stats.badpack;
-            notify(LOG_WARNING, _("Bogus packet (magic is 0x%02X)!\n"),
-             packet.magic);
+            if(rate_limit(C_BADPACKET, 2))
+                notify(LOG_WARNING, _("Bogus packet (magic is 0x%02X)!\n"),
+                 packet.magic);
             continue;
         }
 
@@ -221,11 +220,11 @@ static void mainloop(int fd) {
             continue;
         }
 
-        if(!packet_process(&packet, tty, fd)) {
+        if(!packet_process(&packet, tty, fd))
             /* packet_process always succeeds, but it returns 0 to indicate if
             it wants to skip the payload. */
             G_skip(fd, packet.size, 0);
-        }
+
         pthread_mutex_unlock(&Ttys_lock);
     }
 
@@ -245,9 +244,8 @@ static int packet_preprox(struct rpldev_packet *packet) {
         [EVT_max]    = NULL,
     };
 
-    if(packet->event < EVT_max && tab[packet->event] != NULL) {
+    if(packet->event < EVT_max && tab[packet->event] != NULL)
         ++*tab[packet->event];
-    }
 
     // General packet classification (first stage drop)
     switch(packet->event) {
@@ -270,8 +268,9 @@ static int packet_preprox(struct rpldev_packet *packet) {
             Stats.out += packet->size;
             break;
         default:
-            notify(LOG_WARNING, _("Unknown packet type 0x%02X\n"),
-             packet->event);
+            if(rate_limit(C_PKTTYPE, 2))
+                notify(LOG_WARNING, _("Unknown packet type 0x%02X\n"),
+                 packet->event);
             return 0;
     }
 
@@ -330,57 +329,88 @@ static int evt_open(struct rpldev_packet *packet, struct tty *tty, int fd) {
     - Find out about UID changes on the device and start a new logfile if owner
       has changed.
     */
+    char *sdev = alloca(packet->size + 1);
+    int owner_changed = 0, fill_it = 0;
     struct stat sb;
-    int rv = 0;
 
-    if(tty->sdev == NULL) {
-        char *buf = alloca(packet->size + 1);
-        read(fd, buf, packet->size);
-        buf[packet->size] = '\0';
-        rv = 1;
-        fill_info(tty, buf);
-    }
+    read(fd, sdev, packet->size);
+    sdev[packet->size] = '\0';
+
+    if(tty->sdev == NULL)
+        fill_it = 1;
 
     if(tty->uid != -1 && tty->full_dev != NULL &&
      stat(tty->full_dev, &sb) == 0 && sb.st_uid != tty->uid) {
-        // Create new logfile
-        tty->in = tty->out = 0;
-        log_open(tty);
+        // Create new logfile if owner changed
+        tty->in       = tty->out = 0;
+        owner_changed = 1;
+        fill_it       = 1;
     }
 
-    return rv;
+    if(fill_it)
+        fill_info(tty, sdev);
+    if(owner_changed)
+        log_open(tty);
+
+    return 1;
 }
 
 static void log_open(struct tty *tty) {
-    struct rpldsk_packet p = {
-        .event = EVT_ID_PROG,
-        .magic = MAGIC_SIG,
-        .time  = {-1, -1},
-    };
+    struct rpldsk_packet p = {.magic = MAGIC_SIG, .time = {-1, -1}};
+    struct tm now_tm, *nowp;
     char buf[MAXFNLEN];
+    time_t now_sec;
     size_t s;
 
-    if(check_parent_directory(tty->log) <= 0) {
+    if(check_parent_directory(tty->log) <= 0 && rate_limit(C_LOGOPEN, 5))
         notify(LOG_ERR, _("Directory permission denied: It won't be possible"
          " to write to the file %s, expect warnings.\n"), tty->log);
-    }
+
+    if(tty->fd >= 0)
+        close(tty->fd);
 
     tty->fd = open(tty->log, O_WRONLY | O_CREAT | O_APPEND, S_IWUSR);
 
-    /* Add an optional ident header to record the version of which this logfile
-    was created with. Also, add the full path name of the tty for reference. */
+    /* Add an optional ident header to record the program and version which
+    this logfile was created with. */
     strncpy(buf, "ttyrpld " TTYRPLD_VERSION, sizeof(buf) - 1);
     buf[sizeof(buf) - 1] = '\0';
+    p.event = EVT_ID_PROG;
     s = p.size = strlen(buf);
-    SWAB(&p.size, sizeof(p.size));
+    SWAB1(&p.size);
     write(tty->fd, &p, sizeof(struct rpldsk_packet));
     write(tty->fd, buf, s);
 
+    // Also add the timestamp this log was created
+    now_sec = time(NULL);
+#if defined(__OpenBSD__)
+    nowp = localtime(&now_sec);
+#else
+    nowp = localtime_r(&now_sec, &now_tm);
+#endif
+    strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M:%S", nowp);
+    p.event = EVT_ID_TIME;
+    s = p.size = strlen(buf);
+    SWAB1(&p.size);
+    write(tty->fd, &p, sizeof(struct rpldsk_packet));
+    write(tty->fd, buf, s);
+
+    // ... and the full path name of the tty for reference.
     p.event = EVT_ID_DEVPATH;
     s = p.size = strlen(tty->full_dev);
-    SWAB(&p.size, sizeof(p.size));
+    SWAB1(&p.size);
     write(tty->fd, &p, sizeof(struct rpldsk_packet));
     write(tty->fd, tty->full_dev, s);
+
+    // ... as well as the username (or UID) the tty belongs to
+    if(getnamefromuid(tty->uid, buf, sizeof(buf)) == NULL)
+        snprintf(buf, sizeof(buf), "%ld", (long)tty->uid);
+    p.event = EVT_ID_USER;
+    s = p.size = strlen(buf);
+    SWAB1(&p.size);
+    write(tty->fd, &p, sizeof(struct rpldsk_packet));
+    write(tty->fd, buf, s);
+
     return;
 }
 
@@ -388,13 +418,16 @@ static void log_write(struct rpldev_packet *packet, struct tty *tty, int fd) {
     char *buffer = alloca(packet->size);
     int have;
 
-    if(tty->fd < 0) { log_open(tty); }
-    if((have = read(fd, buffer, packet->size)) <= 0) { return; }
-    if(have != packet->size) { packet->size = have; }
+    if(tty->fd < 0)
+        log_open(tty);
+    if((have = read(fd, buffer, packet->size)) <= 0)
+        return;
+    if(have != packet->size)
+        packet->size = have;
 
-    SWAB(&packet->size, sizeof(packet->size));
-    SWAB(&packet->time.tv_sec, sizeof(packet->time.tv_sec));
-    SWAB(&packet->time.tv_usec, sizeof(packet->time.tv_usec));
+    SWAB1(&packet->size);
+    SWAB1(&packet->time.tv_sec);
+    SWAB1(&packet->time.tv_usec);
     write(tty->fd, &packet->size, sizeof(struct rpldsk_packet));
     write(tty->fd, buffer, have);
     return;
@@ -403,24 +436,25 @@ static void log_write(struct rpldev_packet *packet, struct tty *tty, int fd) {
 //-----------------------------------------------------------------------------
 static int check_parent_directory(const char *s) {
     char *path = alloca(strlen(s) + 1), *p;
+
     strcpy(path, s);
-    if((p = strrchr(path, '/')) == NULL) {
+    if((p = strrchr(path, '/')) == NULL)
         // Current dirctory, no more dir checks needed
         return 1;
-    }
+
     *p = '\0';
     return HX_mkdir(path); // like `mkdir -p`
 }
 
 static void fill_info(struct tty *tty, const char *aux_sdev) {
     char full_dev[MAXFNLEN], sdev[MAXFNLEN], buf[MAXFNLEN],
-     fmday[16], fmtime[16], user[32];
-    struct HX_repmap catalog[] = {
-        {'d', NULL, fmday},
-        {'l', NULL, sdev},
-        {'t', NULL, fmtime},
-        {'u', NULL, user},
-        {0}
+     fmday[16], fmtime[16], user[64];
+    struct HXoption catalog[] = {
+        {.sh = 'd', .type = HXTYPE_STRING, .ptr = fmday},
+        {.sh = 'l', .type = HXTYPE_STRING, .ptr = sdev},
+        {.sh = 't', .type = HXTYPE_STRING, .ptr = fmtime},
+        {.sh = 'u', .type = HXTYPE_STRING, .ptr = user},
+        HXOPT_TABLEEND,
     };
     const char *pbase = NULL;
     struct stat sb;
@@ -441,12 +475,9 @@ static void fill_info(struct tty *tty, const char *aux_sdev) {
         }
         if(pbase == NULL && *full_dev == '/') { pbase = ""; }
     } else if(!find_devnode(tty->dev, full_dev, sizeof(full_dev), &pbase)) {
-        /* Tricky code. find_devnode() is always called [if aux_sdev == NULL],
-        and this branch containing the sprintf is only executed if no devnode
-        was found! In that case, we use [MAJOR:MINOR] as a fictitious filename.
-        Of course, we will not be able to get the UID then. */
-        snprintf(full_dev, sizeof(full_dev),
-         "[%lu:%lu]", KD26_PARTS(tty->dev));
+        /* Use [MAJOR:MINOR] as a fictitious filename if the device node
+        could not be found. */
+        snprintf(full_dev, sizeof(full_dev), "[%lu:%lu]", KD26_PARTS(tty->dev));
     }
 
     /* rpld is able to sort logs by user (by putting each user's logfiles into
@@ -456,10 +487,9 @@ static void fill_info(struct tty *tty, const char *aux_sdev) {
         strcpy(user, _("NONE"));
     } else {
         tty->uid = sb.st_uid;
-        if(getnamefromuid(sb.st_uid, user, sizeof(user)) == NULL) {
-            // Well, at least a UID.
-            snprintf(user, sizeof(user), "%d", sb.st_uid);
-        }
+        if(getnamefromuid(sb.st_uid, user, sizeof(user)) == NULL)
+            // Well, at least the UID.
+            snprintf(user, sizeof(user), "%ld", (long)sb.st_uid);
     }
 
     /* The filename in sdev contains a common prefix, such as "/dev", and may
@@ -467,32 +497,33 @@ static void fill_info(struct tty *tty, const char *aux_sdev) {
     filename cannot contain a slash -- it would otherwise always be treated as
     another directory component. */
     if(pbase != NULL) {
-        /* +1 in source argument: do not copy 2nd slash in e.g. "/dev/tty1"
-        no -1 in length argument: _do_ copy trailing '\0'. */
-        memmove(sdev, full_dev + strlen(pbase) + 1,
-         strlen(full_dev) - strlen(pbase));
+        memmove(sdev, full_dev + strlen(pbase) + 1, // only copy "pts/2" part
+         strlen(full_dev) - strlen(pbase));         // copy includes '\0'
     } else {
         // Usually this is [MAJOR:MINOR]
         strncpy(sdev, full_dev, sizeof(sdev) - 1);
         sdev[sizeof(sdev) - 1] = '\0';
     }
     while(i < sizeof(sdev)) {
-        if(sdev[i] == '/') { sdev[i] = '-'; }
+        if(sdev[i] == '/')
+            sdev[i] = '-';
         ++i;
     }
 
-    // Keep the device path that was used first
+    /* Keep the device path that was used first. I do this because the
+    major-minor may stay the same, but the device node may be named
+    differently. (I know this does not happen, but let's have it.) */
     if(tty->sdev == NULL) {
         HX_strclone(&tty->sdev, sdev);
         HX_strclone(&tty->full_dev, full_dev);
     }
 
     { // What would be a logfile without a proper timestamp?
-      time_t now = time(NULL);
-      struct tm now_tm;
-      localtime_r(&now, &now_tm);
-      strftime(fmday,  sizeof(fmday),  "%Y%m%d", &now_tm);
-      strftime(fmtime, sizeof(fmtime), "%H%M%S", &now_tm);
+        time_t now = time(NULL);
+        struct tm now_tm;
+        localtime_r(&now, &now_tm);
+        strftime(fmday,  sizeof(fmday),  "%Y%m%d", &now_tm);
+        strftime(fmtime, sizeof(fmtime), "%H%M%S", &now_tm);
     }
 
     HX_strrep5(GOpt.ofmt, catalog, buf, sizeof(buf));
@@ -508,23 +539,23 @@ static int init_device(const char *in_devs) {
     while((devp = HX_strsep(&workp, ":")) != NULL) {
         if(devp[0] == '-' && devp[1] == '\0') {
             fd = STDIN_FILENO;
-            if(GOpt.verbose) { printf(_("Connected to %s\n"), "<stdin>"); }
+            if(GOpt.verbose)
+                printf(_("Connected to %s\n"), "<stdin>");
             break;
         }
         if((fd = open(devp, O_RDONLY)) >= 0) {
-            if(GOpt.verbose) { printf(_("Connected to %s\n"), devp); }
+            if(GOpt.verbose)
+                printf(_("Connected to %s\n"), devp);
             break;
         }
-        if(errno != ENOENT) {
+        if(errno != ENOENT)
             fprintf(stderr, _("static_find(): Could not open %s even"
              " though it exists: %s (trying next device)\n"),
              devp, strerror(errno));
-        }
-        if(errno == EBUSY) {
+        else if(errno == EBUSY)
             fprintf(stderr, _("\t" "The RPL device can only be opened"
              " once,\n\t" "there is probably an instance of rpld"
              " running!\n"));
-        }
     }
 
     free(copy);
@@ -534,9 +565,8 @@ static int init_device(const char *in_devs) {
 static int init_sbuffer(int fd) {
     int eax = -1;
 
-    if(D_IOCTL(fd, RPL_IOC_GETBUFSIZE, eax) && eax > 0) {
+    if(D_IOCTL(fd, RPL_IOC_GETBUFSIZE, &eax) && eax > 0)
         Opt.bsize = eax;
-    }
 
 #if defined(__FreeBSD__) || defined(__OpenBSD__)
     if((Buffer = malloc(Opt.bsize)) == NULL)
@@ -587,15 +617,13 @@ static int init_sighandler(void) {
 
 static void sighandler_int(int s) {
     if(Opt._running-- == 0) {
-        if(GOpt.verbose) {
+        if(GOpt.verbose)
             printf(_("Second time we received SIGINT/SIGTERM,"
              " exiting immediately.\n"));
-        }
         exit(EXIT_FAILURE);
     }
-    if(GOpt.verbose) {
+    if(GOpt.verbose)
         printf(_("\n" "Received SIGINT/SIGTERM, shutting down.\n"));
-    }
     Opt._running = 0;
     return;
 }
@@ -606,7 +634,8 @@ static void sighandler_alrm(int s) {
       Stats.read, Stats.write, Stats.in, Stats.out, Stats.ioctl, Stats.badpack
     );
     fflush(stdout);
-    if(GOpt.verbose) { alarm(1); }
+    if(GOpt.verbose)
+        alarm(1);
     return;
 }
 
@@ -615,80 +644,6 @@ static void sighandler_pipe(int s) {
     return;
 }
 
-//-----------------------------------------------------------------------------
-static int get_options(int *argc, const char ***argv) {
-    const struct HXoption options_table[] = {
-        {.sh = 'D', .type = HXOPT_STRING, .ptr = &Opt.device,
-         .help = _("Path to the RPL device"), .htyp = _("file")},
-        {.sh = 'I', .type = HXOPT_VAL, .ptr = &Opt.infod_start, .val = 1,
-         .help = _("Start INFOD subcomponent")},
-        {.sh = 'O', .type = HXOPT_STRING, .ptr = &GOpt.ofmt,
-         .help = _("Override OFMT variable"), .htyp = _("string")},
-        {.sh = 'Q', .type = HXOPT_NONE, .ptr = &Opt.dolog, .val = 0,
-         .help = _("Deactivate logging, only do bytecounting")},
-        {.sh = 'U', .type = HXOPT_STRING, .cb = getopt_username,
-         .help = _("User to change to"), .htyp = _("user")},
-        {.sh = 'c', .type = HXOPT_STRING, .cb = getopt_config,
-         .help = _("Read configuration from file"),
-         .htyp = _("file")},
-        {.sh = 'i', .type = HXOPT_VAL, .ptr = &Opt.infod_start, .val = 0,
-         .help = _("Do not start INFOD subcomponent")},
-        {.sh = 's', .type = HXOPT_NONE, .ptr = &GOpt.syslog,
-         .help = _("Print warnings/errors to syslog")},
-        {.sh = 'v', .type = HXOPT_NONE | HXOPT_INC, .ptr = &GOpt.verbose,
-         .help = _("Print statistics while rpld is running (overrides -s)")},
-        HXOPT_AUTOHELP,
-        HXOPT_TABLEEND,
-    };
-
-    return HX_getopt(options_table, argc, argv, HXOPT_USAGEONERR);
-}
-
-static void getopt_config(const struct HXoptcb *cbi) {
-    read_config(cbi->s);
-    return;
-}
-
-static void getopt_username(const struct HXoptcb *cbi) {
-    rdconfig_user(NULL, cbi->s);
-    return;
-}
-    
-static int read_config(const char *file) {
-    static struct shconf_opt config_table[] = {
-        {"DEVICE", SHCONF_STRING, &Opt.device},
-        {"OFMT",   SHCONF_STRING, &GOpt.ofmt},
-        {"DO_LOG", SHCONF_IBOOL,  &Opt.dolog},
-        {"USER",   SHCONF_CB,     &GOpt.user_id, rdconfig_user},
-
-        {"INFOD_PORT", SHCONF_STRING, &GOpt.infod_port},
-        {"START_INFOD", SHCONF_IBOOL, &Opt.infod_start},
-        {NULL},
-    };
-    return HX_shconfig(file, config_table);
-}
-
-static int read_config_bp(const char *app_path, const char *file) {
-    char *fpath = HX_strdup(app_path), *ptr, construct[MAXFNLEN];
-    if((ptr = strrchr(fpath, '/')) == NULL) {
-        construct[MAXFNLEN - 1] = '\0';
-        strncpy(construct, file, MAXFNLEN - 1);
-    } else {
-        *ptr++ = '\0';
-        snprintf(construct, MAXFNLEN, "%s/%s", fpath, file);
-    }
-    free(fpath);
-    return read_config(construct);
-}
-
-static int rdconfig_user(const struct shconf_opt *t, const char *val) {
-    if((GOpt.user_id = getuidfromname(val)) < 0) {
-        fprintf(stderr, _("No such user: %s\n"), val);
-        exit(EXIT_FAILURE);
-    }
-    return 1;
-}
-
 //-----------------------------------------------------------------------------
 static int find_devnode(uint32_t id, char *dest, size_t len,
  const char **loc_pbase)
@@ -699,7 +654,8 @@ static int find_devnode(uint32_t id, char *dest, size_t len,
     const char **dirp = Device_dirs;
     while(*dirp != NULL) {
         if(find_devnode_dive(id, dest, len, *dirp)) {
-            if(loc_pbase != NULL) { *loc_pbase = *dirp; }
+            if(loc_pbase != NULL)
+                *loc_pbase = *dirp;
             return 1;
         }
         ++dirp;
@@ -719,10 +675,13 @@ static int find_devnode_dive(uint32_t id, char *dest, size_t len,
     void *dx;
     char *de;
 
-    if((dx = HXdir_open(dir)) == NULL) { return 0; }
+    if((dx = HXdir_open(dir)) == NULL)
+        return 0;
+
     while((de = HXdir_read(dx)) != NULL) {
         snprintf(buf, sizeof(buf), "%s/%s", dir, de);
-        if(*de == '.' || stat(buf, &sb) != 0) { continue; }
+        if(*de == '.' || stat(buf, &sb) != 0 || S_ISLNK(sb.st_mode))
+            continue;
         if(S_ISCHR(sb.st_mode) &&
          K26_MKDEV(COMPAT_MAJOR(sb.st_rdev), COMPAT_MINOR(sb.st_rdev)) == id) {
             strncpy(dest, buf, len);
@@ -732,9 +691,11 @@ static int find_devnode_dive(uint32_t id, char *dest, size_t len,
         }
         if(S_ISDIR(sb.st_mode)) {
             snprintf(buf, sizeof(buf), "%s/%s", dir, de);
-            if(find_devnode_dive(id, dest, len, buf)) { return 1; }
+            if(find_devnode_dive(id, dest, len, buf))
+                return 1;
         }
     }
+
     HXdir_close(dx);
     return 0;
 }
@@ -749,7 +710,8 @@ static char *getnamefromuid(uid_t uid, char *result, size_t len) {
     char additional[1024];
     getpwuid_r(uid, &ent, additional, sizeof(additional), &ep);
 #endif
-    if(ep == NULL) { return NULL; }
+    if(ep == NULL)
+        return NULL;
     strncpy(result, ep->pw_name, len - 1);
     result[len - 1] = '\0';
     return result;
@@ -764,8 +726,89 @@ static uid_t getuidfromname(const char *name) {
     char additional[1024];
     getpwnam_r(name, &ent, additional, sizeof(additional), &ep);
 #endif
-    if(ep == NULL) { return -1; }
+    if(ep == NULL)
+        return -1;
     return ep->pw_uid;
 }
 
+static int get_options(int *argc, const char ***argv) {
+    const struct HXoption options_table[] = {
+        {.sh = 'D', .type = HXTYPE_STRING, .ptr = &Opt.device,
+         .help = _("Path to the RPL device"), .htyp = _("file")},
+        {.sh = 'I', .type = HXTYPE_VAL, .ptr = &Opt.infod_start, .val = 1,
+         .help = _("Start INFOD subcomponent")},
+        {.sh = 'O', .type = HXTYPE_STRING, .ptr = &GOpt.ofmt,
+         .help = _("Override OFMT variable"), .htyp = _("string")},
+        {.sh = 'Q', .type = HXTYPE_NONE, .ptr = &Opt.dolog, .val = 0,
+         .help = _("Deactivate logging, only do bytecounting")},
+        {.sh = 'U', .type = HXTYPE_STRING, .cb = getopt_username,
+         .help = _("User to change to"), .htyp = _("user")},
+        {.sh = 'c', .type = HXTYPE_STRING, .cb = getopt_config,
+         .help = _("Read configuration from file"),
+         .htyp = _("file")},
+        {.sh = 'i', .type = HXTYPE_VAL, .ptr = &Opt.infod_start, .val = 0,
+         .help = _("Do not start INFOD subcomponent")},
+        {.sh = 's', .type = HXTYPE_NONE, .ptr = &GOpt.syslog,
+         .help = _("Print warnings/errors to syslog")},
+        {.sh = 'v', .type = HXTYPE_NONE | HXOPT_INC, .ptr = &GOpt.verbose,
+         .help = _("Print statistics while rpld is running (overrides -s)")},
+        HXOPT_AUTOHELP,
+        HXOPT_TABLEEND,
+    };
+
+    return HX_getopt(options_table, argc, argv, HXOPT_USAGEONERR);
+}
+
+static void getopt_config(const struct HXoptcb *cbi) {
+    read_config(cbi->s);
+    return;
+}
+
+static void getopt_username(const struct HXoptcb *cbi) {
+    if((GOpt.user_id = getuidfromname(cbi->s)) < 0) {
+        fprintf(stderr, _("No such user: %s\n"), cbi->s);
+        exit(EXIT_FAILURE);
+    }
+    return;
+}
+    
+static int rate_limit(int counter, time_t delta) {
+    static time_t last_time[C_MAX] = {};
+    time_t now = time(NULL);
+
+    if(now > last_time[counter] + delta) {
+        last_time[counter] = now;
+        return 1;
+    }
+
+    return 0;
+}
+
+static int read_config(const char *file) {
+    static struct HXoption config_table[] = {
+        {.ln = "DEVICE",      .type = HXTYPE_STRING, .ptr = &Opt.device},
+        {.ln = "DO_LOG",      .type = HXTYPE_BOOL,   .ptr = &Opt.dolog},
+        {.ln = "INFOD_PORT",  .type = HXTYPE_STRING, .ptr = &GOpt.infod_port},
+        {.ln = "START_INFOD", .type = HXTYPE_BOOL,   .ptr = &Opt.infod_start},
+        {.ln = "OFMT",        .type = HXTYPE_STRING, .ptr = &GOpt.ofmt},
+        {.ln = "USER",        .type = HXTYPE_NONE,   .ptr = &GOpt.user_id,
+         .cb = getopt_username},
+        {NULL},
+    };
+    return HX_shconfig(file, config_table);
+}
+
+static int read_config_bp(const char *app_path, const char *file) {
+    char *fpath = HX_strdup(app_path), *ptr, construct[MAXFNLEN];
+    if((ptr = strrchr(fpath, '/')) == NULL) {
+        strncpy(construct, file, MAXFNLEN - 1);
+        construct[sizeof(construct) - 1] = '\0';
+    } else {
+        *ptr++ = '\0';
+        snprintf(construct, sizeof(construct), "%s/%s", fpath, file);
+    }
+    free(fpath);
+    return read_config(construct);
+}
+
 //=============================================================================
